import * as React from 'react';
import classNames from 'classnames';
import List from '@material-ui/core/List';
import { makeStyles } from '@material-ui/styles';
import Drawer from '@material-ui/core/Drawer';
import LystDrawerItem from '../LystDrawerItem';
import LystDrawerSubItem from '../LystDrawerSubItem';
import LystCollapsible from '../LystCollapsible';
import { PatientMenu } from '../../types';
import LystDropDown from '../LystDropDown';
import LystDropDownItem from '../LystDropDownItem';
import { drawerHook } from '../../customs/hooks';
// Default DrawerStyles for collapse/expand animation
// keys for Inline styles must be camelcase
// check allowed types for each key by hovering over or (cmd + 'click') on specific key
const drawerStyles = makeStyles((theme: any) => ({
  root: {
    display: 'flex',
    height: '100%',
    overflow: 'auto',
    overflowX: 'hidden'
  },
  drawerOpen: {
    width: 235,
    transition: theme.transitions.create('width', {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.enteringScreen
    })
  },
  drawerClose: {
    transition: theme.transitions.create('width', {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.leavingScreen
    }),
    overflowX: 'hidden',
    width: theme.spacing.unit * 6 + 1,
    [theme.breakpoints.up('sm')]: {
      width: theme.spacing.unit * 7.5 + 1
    }
  }
}));

const LystDrawer: React.FunctionComponent<{
  open: boolean;
  menuItems: PatientMenu[];
  position: 'left' | 'right';
  styles?: any;
}> = ({ position, open, menuItems, styles }) => {
  const drawerClasses = drawerStyles();
  const {
    selectedIndex,
    anchorElement,
    handleListItemClick,
    handleClose
  } = drawerHook(open);

  return (
    <aside className={drawerClasses.root}>
      <Drawer
        anchor={position}
        variant={'permanent'}
        classes={{
          paper: classNames({
            [drawerClasses.drawerOpen]: open,
            [drawerClasses.drawerClose]: !open
          })
        }}
        open={open}
      >
        <List disablePadding={true}>
          {menuItems.map((rootItem, index, array) => (
            // mapping through menuItems Array
            // style divider for all except last item
            // Pass array index and the state selectedIndex to only expand subItems that belong to the event(click) target
            <div key={`rootItem-${index}`}>
              <LystDrawerItem
                placement={position}
                tooltip={!open}
                tabIndex={index}
                styles={styles.drawerItem}
                name={rootItem.name}
                handleClick={handleListItemClick}
                Icon={rootItem.icon}
                divider={array.length - 1 !== index}
              />
              <LystCollapsible open={open && index === selectedIndex}>
                {rootItem.subItems.map((subItem, index) => (
                  // Pass name and path to the navlink component
                  <LystDrawerSubItem
                    key={`subItem-${index}`}
                    content={subItem.name}
                    path={subItem.path}
                    styles={styles.drawerSubItem}
                  />
                ))}
              </LystCollapsible>
              {!open && index === selectedIndex && (
                // Only Render LystDropDown when Drawer is closed and the item selected matches corresponding index in menuItems array
                // anchorElement is the target node to enable position dropDown near the element
                <LystDropDown
                  position={position}
                  handleClose={handleClose}
                  anchorElement={anchorElement}
                  styles={styles.dropDown}
                >
                  <LystDropDownItem content={rootItem.name} />
                  {rootItem.subItems.map((subItem, index) => (
                    <LystDropDownItem
                      key={`dropDown-${index}`}
                      component={subItem.component}
                      content={subItem.name}
                    />
                  ))}
                </LystDropDown>
              )}
            </div>
          ))}
        </List>
      </Drawer>
    </aside>
  );
};

LystDrawer.defaultProps = {
  styles: {
    drawerSubItem: null,
    drawerItem: null,
    dropDown: null
  }
};

export default LystDrawer;
